/*
 * File: songs.h
 * -------------
 * This file contains definitions for the various data types we'll be using
 * to implement our song library. Both Song and Library are defined here since
 * they are so closely related. Any file which includes "song.h" will pick up
 * both definitions.
 */

#ifndef SONGS_H_
#define SONGS_H_

#include <string>
#include "vector.h"
#include "map.h"

// The Song struct provides us a convenient way to package data together. It
// also emphasizes that we want to think of a Song as a single entity, not as
// three strings and an integer.
//
// structs and classes are extremely similar in C++. In fact, the only
// difference is that all data/methods are public by default in a struct and
// private by default in a class. However, we tend to use structs to represent
// simple data types that usually don't have any methods, and classes for
// anything more complex.
struct Song {
    std::string artist;
    std::string album;
    std::string title;
    int rating;
};

// The Library class provides the management tools for our songs. It internally
// stores the songs in a way that provides quick lookup of songs by artist or
// by album. Clients should not be concerned with how the songs are actually
// stored; instead, they should only care about the public interface that
// Library exposes.
class Library {
public:
    // The public section of the class consists of all things that are part of
    // the public interface of our class. However, we only list the prototypes
    // here; the implementation details are hidden away in songs.cpp. Clients
    // should only know how to use these methods, not how they are implemented.

    // The following two "methods" are actually called constructors -- they're
    // called whenever a new Library object is created. Constructors are used
    // to initialize the internal data of the Library. The first one is called
    // a default constructor, since it takes no arguments. It is invoked in the
    // following scenario:
    //
    //     Library library;
    //
    // The second constructor takes in a single argument, the name of a file
    // containing song data. It is invoked in the following scenario:
    //
    //     Library library("song-data.txt");
    //
    Library();
    Library(std::string fileName);

    // addSong adds the specified song to the library.
    void addSong(Song s);

    // getAllSongs returns all songs stored in the library in a single list.
    Vector<Song> getAllSongs();

    // getSongsByArtist returns all songs associated with the specified artist
    // in a single list. getSongsByAlbum does the same for songs associated
    // with the specified album name. Both are implemented to be case-
    // insensitive.
    Vector<Song> getSongsByArtist(std::string artist);
    Vector<Song> getSongsByAlbum(std::string album);

private:
    // The private section of a class provides implementation details the
    // client doesn't need to know about. Usually it contains instance
    // variables, but it can also include any private helper functions used
    // by the rest of the class.

    // artistMap contains artist names as keys and lists of associated songs
    // as values. albumMap contains album names as keys instead. Note that this
    // is not very space efficient; each song is stored twice (once in each
    // map). As a challenge, think of a way to modify the implementation to be
    // both time-efficient for lookup by artist or albums, and space-efficient
    // in how it stores the data.
    Map<std::string, Vector<Song> > artistMap;
    Map<std::string, Vector<Song> > albumMap;
};

#endif
